#ifndef PATCHDOWNLOADER_H
#define PATCHDOWNLOADER_H

#include <QObject>
#include <QTimer>
#include <QDate>
#include <QDir>
#include <QSettings>
#include <QMap>
#include "models/downloader.h"

struct Patch {
    QUrl url;
    QString md5_hash;
    QString datetime;
};

class PatchDownloader : public QObject
{
    Q_OBJECT
public:
    explicit PatchDownloader(QObject *parent = nullptr);
    ~PatchDownloader();
    void enableAutoUpdate();
    void disableAutoUpdate();
    void stopAllDownloads();
    void startAllDownloads();

public slots:
    void checkForUpdates();

private slots:
    void onDownloaderCompleted(Downloader* downloader_ptr);
    void onDownloaderProgressChanged(Downloader* downloader_ptr);

signals:
    void getPatchListError();
    void removeFileError(QString filename);
    void downloadNewFilesError();

    void downloadCompleted();
    void downloadStarted();
    void progressChanged(quint64 bytesDownloaded, quint64 bytesTotal,
                         QString download_speed_formatted,
                         QString elapsed_time_formatted);

    void patchDateChanged(QString patch_name, QDate patch_date);

private:
    int versionFromPatchFilename(QString filename);
    bool updatePatchList();
    bool removeOldPatchesFromDirecrory();
    bool addMissingPatchesToDownloadList();
    void clearDownloadQueue();

private:
    QTimer auto_update_timer;

    QDir patch_download_dir;
    QList<Patch> patches;

    QTimer update_check_timer;
    QList<Downloader*> download_queue;
};

#endif // PATCHDOWNLOADER_H
